package com.fjhx.sale.service.arrival.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.sale.entity.arrival.dto.ArrivalDetailDto;
import com.fjhx.sale.entity.arrival.dto.ArrivalDetailSelectDto;
import com.fjhx.sale.entity.arrival.po.ArrivalDetail;
import com.fjhx.sale.entity.arrival.vo.ArrivalDetailVo;
import com.fjhx.sale.mapper.arrival.ArrivalDetailMapper;
import com.fjhx.sale.service.arrival.ArrivalDetailService;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * purchase_到货明细单 服务实现类
 * </p>
 *
 * @author
 * @since 2024-02-01
 */
@Service
public class ArrivalDetailServiceImpl extends ServiceImpl<ArrivalDetailMapper, ArrivalDetail> implements ArrivalDetailService {

    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;
    @Autowired
    private StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    private SupplierInfoService supplierInfoService;
    @Autowired
    private ISysDeptService deptService;

    @Override
    public Page<ArrivalDetailVo> getPage(ArrivalDetailSelectDto dto) {
        IWrapper<ArrivalDetail> wrapper = getWrapper();
        wrapper.orderByDesc("ad", ArrivalDetail::getId);
        Page<ArrivalDetailVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ArrivalDetailVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值供应商
        supplierInfoService.attributeAssign(records, ArrivalDetailVo::getSupplierId, (item, supplierInfo) -> {
            item.setSupplierName(supplierInfo.getName());
        });
        //获取归属公司信息
        Map<Long, String> deptMap = deptService.list().stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //赋值数据
        for (ArrivalDetailVo record : records) {
            //赋值归属公司信息
            record.setCompanyName(deptMap.get(record.getCompanyId()));
        }
        return page;
    }

    @Override
    public void editAccountPeriods(ArrivalDetailDto dto) {
        this.updateById(dto);
        //更新待入库账期
        stockWaitDetailsService.update(q -> q
                .eq(StockWaitDetails::getArrivalId, dto.getId())
                .set(StockWaitDetails::getAccountPeriod, dto.getAccountPeriod())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        //更新流水账期
        stockJournalDetailsService.update(q -> q
                .eq(StockJournalDetails::getArrivalId, dto.getId())
                .set(StockJournalDetails::getAccountPeriod, dto.getAccountPeriod())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

}
