package com.fjhx.sale.service.contract.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.contract.dto.ContractOutboundInfoDto;
import com.fjhx.sale.entity.contract.dto.ContractOutboundInfoSelectDto;
import com.fjhx.sale.entity.contract.dto.ContractOutboundRecordsDto;
import com.fjhx.sale.entity.contract.po.ContractOutboundInfo;
import com.fjhx.sale.entity.contract.po.ContractOutboundRecords;
import com.fjhx.sale.entity.contract.vo.ContractOutboundInfoVo;
import com.fjhx.sale.entity.contract.vo.ContractOutboundRecordsVo;
import com.fjhx.sale.mapper.contract.ContractOutboundInfoMapper;
import com.fjhx.sale.service.contract.ContractOutboundInfoService;
import com.fjhx.sale.service.contract.ContractOutboundRecordsService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 销售出货信息 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-13
 */
@Service
public class ContractOutboundInfoServiceImpl extends ServiceImpl<ContractOutboundInfoMapper, ContractOutboundInfo> implements ContractOutboundInfoService {

    @Autowired
    private ContractOutboundRecordsService contractOutboundRecordsService;
    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public Page<ContractOutboundInfoVo> getPage(ContractOutboundInfoSelectDto dto) {
        IWrapper<ContractOutboundInfo> wrapper = getWrapper();
        wrapper.keyword(dto.getKeyword(), new SqlField("c.code"));
        Page<ContractOutboundInfoVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        List<ContractOutboundInfoVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值开单人名称
        UserUtil.assignmentNickName(records, ContractOutboundInfo::getBillUserId, ContractOutboundInfoVo::setBillUserName);
        return page;
    }

    @Override
    public void loadTruck(ContractOutboundInfoDto dto) {
        this.updateById(dto);
        List<ContractOutboundRecordsDto> outboundRecordList = dto.getOutboundRecordList();
        contractOutboundRecordsService.updateBatchById(BeanUtil.copyToList(outboundRecordList, ContractOutboundRecords.class));
        ObsFileUtil.saveFile(dto.getFileList(), dto.getId(), 0);
        ObsFileUtil.saveFile(dto.getTruckFileList(), dto.getId(), 1);
    }

    @Override
    public ContractOutboundInfoVo detail(Long id) {
        ContractOutboundInfoVo detail = baseMapper.detail(id);
        List<ContractOutboundRecords> list = contractOutboundRecordsService.list(q -> q.eq(ContractOutboundRecords::getRecordId, id));
        List<ContractOutboundRecordsVo> contractOutboundRecordsVos = BeanUtil.copyToList(list, ContractOutboundRecordsVo.class);
        //赋值产品信息
        productInfoService.attributeAssign(contractOutboundRecordsVos, ContractOutboundRecordsVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCustomCode());
            item.setProductName(productInfo.getName());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());

            item.setProductUnit(productInfo.getUnit());
            item.setProductNetWeight(productInfo.getNetWeight());
            item.setProductReverseTexture(productInfo.getReverseTexture());
            item.setProductFrontalTexture(productInfo.getFrontalTexture());
        });

        detail.setOutboundRecordList(contractOutboundRecordsVos);
        return detail;
    }

}
